//----------------------------------------------------------------------------
//
// File:        diskfs.hpp
// Date:        15-Jan-2003
// Programmer:  Marc Rousseau
//
// Description: A class to manage the filesystem information on a TI disk.

// Copyright (c) 2003 Marc Rousseau, All Rights Reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
//
// Revision History:
//
//----------------------------------------------------------------------------

#ifndef DISKFS_HPP_
#define DISKFS_HPP_

// Flags used by TI to indicate file types

#define DATA_TYPE               0x00
#define PROGRAM_TYPE            0x01

#define DISPLAY_TYPE            0x00
#define INTERNAL_TYPE           0x02

#define WRITE_PROTECTED_TYPE    0x08

#define FIXED_TYPE              0x00
#define VARIABLE_TYPE           0x80

const int MAX_FILENAME          = 10;
const int MAX_CHAINS            = 76;

// On-Disk structures used by TI

struct VIB {
    char      VolumeName [ MAX_FILENAME ];
    USHORT    FormattedSectors;
    UCHAR     SectorsPerTrack;
    char      DSK [ 3 ];
    UCHAR     reserved;
    UCHAR     TracksPerSide;
    UCHAR     Sides;
    UCHAR     Density;
    UCHAR     reserved2 [ 36 ]; 
    UCHAR     AllocationMap [ 200 ];
};

struct CHAIN {
    UCHAR     start;
    UCHAR     start_offset;
    UCHAR     offset;
};

struct sFileDescriptorRecord {
    char      FileName [ MAX_FILENAME ];
    char      reserved1 [ 2 ];
    UCHAR     FileStatus;
    UCHAR     RecordsPerSector;
    USHORT    TotalSectors;
    UCHAR     EOF_Offset;
    UCHAR     RecordLength;
    USHORT    NoFixedRecords;               // For some strange reason, this is little-endian!
    char      reserved2 [ 8 ];
    CHAIN     DataChain [ MAX_CHAINS ];
};

struct sSector;

class cFile;
class cDiskMedia;

class cDiskFileSystem {

    static cDiskFileSystem *sm_SortDisk;

    cDiskMedia    *m_Media;
    VIB           *m_VIB;

public:

    cDiskFileSystem ( cDiskMedia * );
    ~cDiskFileSystem ();

    cFile *OpenFile ( const char * );
    cFile *CreateFile ( const char *, UCHAR, int );

    int FreeSectors () const;
    void ShowDirectory () const;
    int GetFilenames ( char *names[] ) const;

    // Routines used by cDiskFile
    sSector *GetFileSector ( sFileDescriptorRecord *FDR, int index );
    int ExtendFile ( sFileDescriptorRecord *FDR, int count );
    void TruncateFile ( sFileDescriptorRecord *FDR, int limit );
    bool AddFile ( cFile * );
    bool DeleteFile ( const char * );
    void DiskModified ();

protected:

    int FindFreeSector ( int start = 0 ) const;
    void SetSectorAllocation ( int index, bool bUsed );

    int FindLastSector ( sFileDescriptorRecord *FDR ) const;
    bool AddFileSector ( sFileDescriptorRecord *FDR, int );

    const sSector *FindSector ( int index ) const;
    sSector *FindSector ( int index );

    int FindFileDescriptorIndex ( const char *name );

    static int sortDirectoryIndex ( const void *ptr1, const void *ptr2 );

    int AddFileDescriptor ( const sFileDescriptorRecord *FDR );

    bool IsValid ( const sFileDescriptorRecord *FDR ) const;

};

inline void cDiskFileSystem::DiskModified () { m_Media->DiskModified (); }

#endif
